#include <libmp/all.h>
#include <libmp/ctx.h>
#include <libmp/error.h>
#include <stdio.h>

#include "../cmd.h"
#include "../log.h"

bool cmd_update_callback(lm_ctx_t *ctx, lm_pkg_t *pkg, char *file, size_t current, size_t total, void *data){
  bar(current, total); 
  return true;
}

bool cmd_update_download_callback(lm_ctx_t *ctx, lm_pkg_t *pkg, bool is_archive, size_t current, size_t total, void *data){
  if(!is_archive)
    bar(current, total);
  return true;
}

bool cmd_update(lm_ctx_t *ctx, config_t *config, args_t *args){
  if(args_get_bool(args, "help")){
    info(_("Listing options for the update command:"));
    printf(_("    "FG_BOLD"--yes"FG_RESET":\tdon't ask for confirmation\n"));
    return true;
  }

  if(lm_ctx_sync(ctx, false, NULL, NULL) <= 0){
    error(_("There are no avaliable pools, have you synced yet?"));
    goto end;
  }

  lm_pkg_t *old = NULL, *new = NULL;
  lm_ctx_update_list_t *list = NULL;
  bool ret = false, r = false;
  ssize_t current = 0;

  if((list = lm_ctx_update_list(ctx)) == NULL){
    error(_("Failed to get the package update list: %s"), lm_strerror());
    goto end;
  }

  if(list->count == 0){
    error(_("All packages are up-to-date"));
    goto end;
  }

  if(list->count == 1)
    info(_("Following %d package will be "FG_BOLD"UPDATED:"FG_RESET), list->count);
  else
    info(_("Following %d packages will be "FG_BOLD"UPDATED:"FG_RESET), list->count);
  
  // list all the packages

  if(!args_get_bool(args, "yes") && !yesno(_("Continue?"))){
    error(_("Operation cancelled"));
    goto end;
  }

  while((old = lm_ctx_update_list_next(list)) != NULL){
    if((new = lm_ctx_update(ctx, old)) == NULL){
      error(_("Failed to update "FG_BOLD"%s"FG_RESET": %s"), old->name, lm_strerror());
      goto end;
    }

    info(_("(%d/%d) Downloading "FG_BOLD"%s"FG_RESET" ("FG_BOLD FG_BLUE"%s"FG_RESET")"), ++current, list->count, new->name, new->version);
    r = lm_ctx_download(ctx, new, cmd_update_download_callback, NULL);
    bar_free();

    if(!r){
      error(_("Failed to download "FG_BOLD"%s"FG_RESET" ("FG_BOLD FG_BLUE"%s"FG_RESET"): %s"), new->name, new->version, lm_strerror());
      goto end;
    }

    info(_("(%d/%d) Removing "FG_BOLD"%s"FG_RESET" ("FG_BOLD FG_BLUE"%s"FG_RESET")"), current, list->count, old->name, old->version);
    r = lm_ctx_remove(ctx, old, cmd_update_callback, NULL);
    bar_free();

    if(!r){
      error(_("Failed to remove "FG_BOLD"%s"FG_RESET" ("FG_BOLD FG_BLUE"%s"FG_RESET"): %s"), old->name, old->version, lm_strerror());
      goto end;
    }

    info(_("(%d/%d) Installing "FG_BOLD"%s"FG_RESET" ("FG_BOLD FG_BLUE"%s"FG_RESET")"), current, list->count, new->name, new->version);
    r = lm_ctx_install(ctx, new, cmd_update_callback, NULL);
    bar_free();

    if(!r){
      error(_("Failed to install "FG_BOLD"%s"FG_RESET" ("FG_BOLD FG_BLUE"%s"FG_RESET"): %s"), new->name, new->version, lm_strerror());
      goto end;
    }
  }

  ret = true;
end:
  lm_ctx_update_list_free(list);
  return ret;
}
